%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Equilibre d'une chaine formee de barres rigides                     %
%%                                                                     %
%% Auteur: J.Ch. Gilbert, Inria                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
warning off all;
format compact;

% Jobs

  job = 1;      % 0: check derivatives
                % 1: optimize

% Variables globales

  global LB        % longueurs desirees des barres
  global GCOL0     % caractere donnant la couleur utilisee dans le graphe de la
                   % chaine initiale
  global GCOL      % caractere donnant la couleur utilisee dans les graphes
  global GTYP0     % caractere donnant le type de ligne dans le graphe de la
                   % chaine initiale
  global GTYP      % caractere donnant le type de ligne dans les graphes
  global HD        % specifie le domaine (ouvert) admissible pour x
                   % 0: Rn, 1: 0<x(i)<1, 2: 0<x(i)<x(i+1)<1, 3: +chaine convexe
  global A B       % coordonnees du dernier noeud fixe
  global PHI PHIP  % description du sol
  global IT        % variables specifiques a CONSTR
  global X0        % position de depart
  global FIG_SIMUL % figures du simulateur
  global FIG_SET
  global FIG_TITLE FIG_INIT FIG_ITER FIG_FOUND
  global FIG_XMIN FIG_XMAX FIG_YMIN FIG_YMAX
  global XYFILE
  global ITER
  global fout

  FIG_SET   = 0;

  FILM_FLAG = 1;
  FILM(1) = getframe(gcf);	% this is just to define the type of FILM before its globalization
  global FILM_FLAG   % 0: do nothing, 1: make a film
  global FILM        % the film obtained from getframe

% OPT201

% data_2a; FIG_ITER = 1;
  data_2b; FIG_ITER = 1;
% data_2c; FIG_ITER = 1;
% data_2d; FIG_ITER = 1;
% data_3a; FIG_ITER = 1;
% data_3b; FIG_ITER = 1;
% data_3c; FIG_ITER = 1;

  ITER=0;
  HD=0;
  simul='chs';		% nom du simulateur

  fout=1;		% sortie sur ecran
% fout=fopen('ch.res','w');		% sortie sur fichier
  XYFILE=fopen('xy.res','w');		% sortie des coordonnees des noeuds

  nb=size(LB,1);   % nombre de barres
  x = xy(:);
  nn=round(size(x,1)/2);       % nombre de noeuds

% Donnees pour l'optimiseur

  maxit  =   20;	% nombre max d'iterations permises
  maxsim = 1000;	% nombre max de simulations permises

% Verification des donnees

  fprintf(fout,'\nDescription de la chaine\n');
  fprintf(fout,'\n  nombre de barres: %i\n',nb);
  if nb < 2,
    fprintf(fout,'\nCH: >>> il faut au moins 2 barres (il y en a %i)\n\n',nb);
    return
  end
  if nb ~= nn+1,
    fprintf(fout,'\nCH: >>> nombre de barres (%i)',nb);
    fprintf(fout,' et de noeuds (%i) incompatibles\n\n',nn);
    return
  end

  l=0;
  for i=1:nb
    if LB(i) <= 0,
      fprintf(fout,'\n\nCH: >>> la barre %i est de longueur negative',i);
      fprintf(fout,' %8.1e\n\n',LB(i));
      return
    end
    l=l+LB(i);
  end
  fprintf(fout,'  longueur totale: %4.1f\n',l);
  if l < sqrt(A^2+B^2),
    fprintf(fout,'\n\nCH: >>> la longueur de la chaine ne suffit pas a relier');
    fprintf(fout,'\n        ses points d''attache\n\n');
    return
  end

% Calcul de f, g, ce et ae

  GCOL0='k';
  GTYP0='-';
  GCOL='m';
  GTYP='--';
  X0=x;
  [f,ce,g,ae,tmp,indic] = feval(simul,4,x);

% Verification des derivees

  if job == 0
    verifderiv (fout,x);
    return
  end

% Optimisation

  if job ~= 1; return; end

  EPSREL=1.e-5;			% seuil d'optimalite
% EPSREL=1.e-14;		% seuil d'optimalite

% if isempty(FIG_SIMUL); FIG_SIMUL=figure; end
  FIG_INIT  = 1;	% pour forcer l'impression de la chaîne initiale
  FIG_SIMUL = 1;
  figure(1);
  clf;
  feval(simul,1,x);
  if indic == -1,
    fprintf(fout,'\nCH: >>> noeuds initiaux hors domaine\n\n')
    return
  end

  options.imode(1) = 1;	% 0: BFGS, 1: Newton, 2: Newton modifié
  options.imode(2) = 0;	% 0: RL d'Armijo, 1: pas unité

  options.tol(1) = 1.e-6;    % tolerance sur le gradient du lagrangien
  options.tol(2) = 1.e-6;    % tolerance sur les contraintes d'egalite
  options.tol(3) = 1.e-6;    % tolerance sur les contraintes d'inegalite
  options.tol(4) = 1.e-6;    % tolerance sur la positivite de multiplicateurs et la complementarite

  options.iter  = maxit;       % max iterations
  options.simul = maxsim;     % max simulations
  options.fout  = fout;        % canal de sortie des impressions
  options.dxmin = 1.e-20;
  options.verb  = 1;

  tic;
  [x,lme,info] = sqp_e (@chs,x,[],options);
  cputime = toc;
  fprintf(fout,'\n\nTemps CPU = %f sec',cputime);

% x final --------------------------------------------------------------

  fprintf(fout,'\n\n  x = [')
  fprintf(fout,'%7.4f ',x(1:nn));
  fprintf(fout,  ' ...\n       ')
  fprintf(fout,'%7.4f ',x(nn+1:2*nn));
  fprintf(fout,  ']'';')

% analyse --------------------------------------------------------------

  FIG_FOUND = true;
  if FIG_FOUND
    FIG_ITER = 1;
    GCOL='b';
    GTYP='-';
    feval(simul,1,x,lme);	% graphe
    print -depsc chainette.eps
%   feval(simul,-2,x,lme);  % analyse de la solution
    if info.flag == 0;
      figure(3);
      clf;
      format long
      ii = info.iter-1;
      quad = info.Fn(2:ii+1)./(info.Fn(1:ii).^2);
      fprintf('\n\nConvergence quadratique: |F(zkp)|/|F(zk)|^2');
      for i=1:ii
        fprintf('\n  %22.15e',quad(i));
      end
      plot([2:info.iter],quad,'--r');
      hold on;
      quad = info.dn(2:info.iter-0)./(info.dn(1:info.iter-1).^2);
      fprintf('\n\nConvergence quadratique: |dkp|/|dk|^2');
      for i=1:ii
        fprintf('\n  %22.15e',quad(i));
      end
      plot([2:info.iter],quad,'r');
      figure(3)
    end
    fprintf('\n');
  end

  if FILM_FLAG
    FILM_FLAG = 2;
    feval(simul,1,x);
%   figure;
%   movie(FILM);
%   movie2avi(FILM,'film.avi');
  end

  fclose(XYFILE);
  fprintf(fout,'\nTemps CPU = %f sec\n',cputime);
